from flask import Blueprint, flash, render_template, request
from flask_wtf import FlaskForm
from wtforms import RadioField
import sys
sys.path.append('~/eval')
from evalweb.web_checker import WebChecker


bp = Blueprint('eval', __name__, url_prefix='/')

tasks = {'DFA': 'DFA',
         'TOT': 'Totální DFA',
         'MIN': 'Minimální DFA',
         'EFA': 'NFA (s ε-kroky)',
         'NFA': 'NFA bez ε-kroků',
         'GRA': 'Regulární gramatika',
         'REG': 'Regulární výraz'}

types = {'DFA': 'DFA',
         'EFA': 'NFA (s ε-kroky)',
         'GRA': 'Regulární gramatika',
         'REG': 'Regulární výraz'}


class TypeForm(FlaskForm):
    make = RadioField('Typ', choices=list(types.items()), default='DFA', coerce=str)


class TaskForm(FlaskForm):
    make = RadioField('Task', choices=list(tasks.items()), default='DFA', coerce=str)


@bp.route('/compare', methods=('GET', 'POST'))
def compare():
    student_form = TaskForm(prefix='student_form')
    teacher_form = TypeForm(prefix='teacher_form')

    if request.method == 'POST':
        teacher_type = teacher_form.make.data
        teacher_string = request.form['teacher_string']
        student_type = student_form.make.data
        student_string = request.form['student_string']

        checker = WebChecker(student_string=student_string, task=student_type)
        task_solved = checker.compare(teacher_string=teacher_string, teacher_type=teacher_type)

        if not checker.ok and checker.eq is not None: # TODO don't know wtf was bad
            extra_word_ce = checker.eq.left_counterexample
            missing_word_ce = checker.eq.right_counterexample
            inf = checker.eq.inf
        else:
            extra_word_ce = None
            missing_word_ce = None
            inf = None

        return render_template('result.html', compare=True, ok=checker.ok, inf=inf, task_solved=task_solved,
                               student_string=student_string, student_type=tasks[student_type],
                               teacher_string=teacher_string, teacher_type=types[teacher_type],
                               extra_word_ce=extra_word_ce, missing_word_ce=missing_word_ce)
        flash(error)

    return render_template('compare.html', student_form=student_form, teacher_form=teacher_form)

@bp.route('/convert', methods=('GET', 'POST'))
def convert():
    student_form = TypeForm(prefix='student_form')
    task_form = TaskForm(prefix='task_form')
    if request.method == 'POST':
        student_string = request.form['student_string']
        student_type = student_form.make.data
        task = task_form.make.data

        webchecker = WebChecker(student_string=student_string, task=task)
        output = webchecker.convert(student_type=student_type)

        return render_template('result.html', compare=False, student_string=student_string,
                               student_type=types[student_type], task=tasks[task], output=output)
        flash(error)

    return render_template('convert.html', student_form=student_form, task_form=task_form)


@bp.route('/userref')
def userref():
    return render_template('userref.html')


@bp.route('/about')
def about():
    return render_template('about.html')